/****************************************************************************
 **
 ** Copyright (C) 2010 Arzel Jrme <myst6re@gmail.com>
 **
 ** This program is free software: you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
 ** the Free Software Foundation, either version 3 of the License, or
 ** (at your option) any later version.
 **
 ** This program is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
 ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
 ** along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ****************************************************************************/

#include "Config.h"

QStringList Config::recentFiles;
QSettings *Config::settings = NULL;
QString Config::_ff8Path;

quint32 Config::sec(const quint32 &time, const int freq_value)
{
	return time%freq(freq_value);
}

quint32 Config::min(const quint32 &time, const int freq_value)
{
	return (time/freq(freq_value))%60;
}

quint32 Config::hour(const quint32 &time, const int freq_value)
{
	return time/(60*freq(freq_value));
}

quint32 Config::time(const quint32 &hour, const quint32 &min, const quint32 &sec, const int freq_value)
{
	int f = freq(freq_value);
	return sec + min*f + hour*f*60;
}

void Config::loadRecentFiles()
{
	QString filePath;
	for(int i=0 ; i<20 ; ++i) {
		filePath = settings->value(QString("recentFile%1").arg(i), QVariant(QString())).toString();
		if(!filePath.isEmpty())
			recentFiles.append(QDir::cleanPath(filePath));
	}
}

void Config::addRecentFile(const QString &filePath)
{
	int index = recentFiles.indexOf(filePath);
	if(index != -1)					recentFiles.removeAt(index);
	recentFiles.prepend(filePath);
	if(recentFiles.size() > 20)		recentFiles.removeLast();
}

QString Config::recentFile(const uint &index)
{
	return recentFiles.at(index);
}

bool Config::hasRecentFiles()
{
	return !recentFiles.isEmpty();
}

int Config::recentFilesSize()
{
	return recentFiles.size();
}

void Config::saveRecentFiles()
{
	int i, recentFilesSize = recentFiles.size();

	for(i=0 ; i<recentFilesSize ; ++i)
		settings->setValue(QString("recentFile%1").arg(i), QVariant(recentFiles.at(i)));

	for(i=recentFilesSize ; i<20 ; ++i)
		settings->setValue(QString("recentFile%1").arg(i), QVariant(QString()));

	settings->sync();
}

void Config::set()
{
	settings = new QSettings(qApp->applicationDirPath()+"/"+PROG_NAME+".ini", QSettings::IniFormat);
}

bool Config::mode()
{
	return settings->value("mode", QVariant(false)).toBool();
}

int Config::freq(const int freq_value)
{
	if(freq_auto())	return freq_value;
	return settings->value("freq", 60).toUInt()==50 ? 50 : 60;
}

bool Config::freq_auto()
{
	return settings->value("freq_auto", QVariant(true)).toBool();
}

QString Config::value(const QString &key, const QString &defaultValue)
{
	return settings->value(key, QVariant(defaultValue)).toString();
}

void Config::setValue(const QString &key, const QVariant &value)
{
	settings->setValue(key, value);
}

QVariant Config::valueVar(const QString &key, const QVariant &defaultValue)
{
	return settings->value(key, defaultValue);
}

QString Config::ff8Path()
{
	if(_ff8Path.isEmpty())
		_ff8Path = QDir::cleanPath( QSettings("Square Soft, Inc", "Final Fantasy VIII").value("1.00/AppPath").toString() );

	return _ff8Path;
}
