/****************************************************************************
 **
 ** Copyright (C) 2010 Arzel Jrme <myst6re@gmail.com>
 **
 ** This program is free software: you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
 ** the Free Software Foundation, either version 3 of the License, or
 ** (at your option) any later version.
 **
 ** This program is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
 ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
 ** along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ****************************************************************************/

#include "Editor.h"

Editor::Editor(QWidget *parent) : QWidget(parent)
{
	liste = new QListWidget(this);
	liste->setFixedWidth(96);
	liste->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
	liste->addItems(QStringList() << tr("G-Forces") << tr("Personnages") << tr("Objets")
					<< tr("Triple Triad") << tr("Chocobo World") << tr("Divers") << tr("Configuration") << tr("HexaEdit"));

	QPushButton *apply = new QPushButton(QIcon(":/images/save.png"), tr("&Sauver"), this);
	QPushButton *cancel = new QPushButton(tr("Annuler"), this);
	apply->setShortcut(QKeySequence(QKeySequence::Save));
	
	stackedLayout = new QStackedLayout;
	
	QGridLayout *gridLayout = new QGridLayout(this);
	gridLayout->addWidget(liste, 0, 0);
	gridLayout->addWidget(apply, 1, 0);
	gridLayout->addWidget(cancel, 2, 0);
	gridLayout->addLayout(stackedLayout, 0, 1, 3, 1);
	
	gfEditor = new GfEditor(this);
	persoEditor = new PersoEditor(this);
	objetEditor = new ItemEditor(this);
	ttriadEditor = new TTriadEditor(this);
	cwEditor = new CWEditor(this);
	diversEditor = new DiversEditor(this);
	configEditor = new ConfigEditor(this);
	allEditor = new AllEditor(this);

	stackedLayout->addWidget(gfEditor);
	stackedLayout->addWidget(persoEditor);
	stackedLayout->addWidget(objetEditor);
	stackedLayout->addWidget(ttriadEditor);
	stackedLayout->addWidget(cwEditor);
	stackedLayout->addWidget(diversEditor);
	stackedLayout->addWidget(configEditor);
	stackedLayout->addWidget(allEditor);
	
	connect(liste, SIGNAL(currentRowChanged(int)), SLOT(setCurrentSection(int)));
	connect(apply, SIGNAL(released()), SLOT(sauver()));
	connect(cancel, SIGNAL(released()), SIGNAL(rejected()));

	liste->setCurrentRow(0);
}

void Editor::setCurrentSection(int id)
{
	if(id==7)
		allEditor->fillPage();
	stackedLayout->setCurrentIndex(id);
}

void Editor::charger(SaveData *saveData)
{
	this->saveData = saveData;
	descCopy = saveData->descData();
	copy = saveData->mainData();

	gfEditor->load(&copy);
	persoEditor->load(&copy, &descCopy);
	objetEditor->load(&copy);
	ttriadEditor->load(&copy);
	cwEditor->load(&copy, &descCopy);
	diversEditor->load(&copy, saveData->freq_value());
	configEditor->load(&copy);
	allEditor->load(&copy, &descCopy);

	updateMode();
	updateTime();
}

void Editor::sauver()
{
	gfEditor->savePage();
	persoEditor->savePage();
	objetEditor->savePage();
	ttriadEditor->savePage();
	cwEditor->savePage();
	diversEditor->savePage();
	configEditor->savePage();

	saveData->setMainData(copy);
	saveData->setDescData(descCopy);

	emit accepted();
}

void Editor::updateMode()
{
	int lastRow = liste->count()-1;
	liste->item(lastRow)->setHidden(!Config::mode());
	if(liste->currentRow()==lastRow && !Config::mode())
		liste->setCurrentRow(0);

	gfEditor->updateMode();
	persoEditor->updateMode();
	// ttriadEditor->updateMode();
	diversEditor->updateMode();
	configEditor->updateMode();
	// allEditor->updateMode();
}

void Editor::updateTime()
{
	// gfEditor->updateTime();
	// persoEditor->updateTime();
	// ttriadEditor->updateTime();
	diversEditor->updateTime();
	// configEditor->updateTime();
	// allEditor->updateTime();
}
