/****************************************************************************
 **
 ** Copyright (C) 2010 Arzel Jrme <myst6re@gmail.com>
 **
 ** This program is free software: you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
 ** the Free Software Foundation, either version 3 of the License, or
 ** (at your option) any later version.
 **
 ** This program is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
 ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
 ** along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ****************************************************************************/

#include "FF8Text.h"

QString FF8Text::toUnicode(QString ff8str)
{
	QString ret;

	foreach(QChar comp, ff8str)
		ret.append(QString::fromUtf8(caract[(quint8)comp.toLatin1()]));

	return ret;
}

QByteArray FF8Text::textToff8(QString string)
{
	QByteArray ff8str;
	int i;

	foreach(QChar comp, string)
	{
		for(i=0x20 ; i<0xe3 ; ++i)
		{
			if(QString::fromUtf8(caract[i]).compare(comp)==0)
			{
				ff8str.append((char)i);
				break;
			}
		}
	}

	return ff8str;
}

char FF8Text::charFromff8(char comp)
{
	QString ret = QString::fromUtf8(caract[(quint8)comp]);
	return ret.isEmpty() ? '.' : ret.toLatin1().at(0);
}

void FF8Text::toPixmap(QPainter *painter, QPoint point, QByteArray ff8str)
{
	int ff8strSize = ff8str.size();
	if(ff8strSize<=0)	return;
	QPixmap fontPixmap(":/images/font"+Config::value("font")+".png");

	quint8 charId;
	quint16 pos = 0;
	for(quint16 i=0 ; i<ff8strSize ; ++i)
	{
		charId = (quint8)ff8str.at(i)-32;
		painter->drawPixmap(point+QPoint(pos, 0), fontPixmap, QRect((charId%21)*24, (charId/21)*24, width[charId], 24));
		pos += width[charId] + 2;
	}
}

void FF8Text::textToPixmap(QPainter *painter, QPoint point, QString string)
{
	FF8Text::toPixmap(painter, point, FF8Text::textToff8(string));
}

QByteArray FF8Text::numToBiosText(quint32 num, quint8 width)
{
	QString chaine = QString("%1").arg(num, width, 10, QChar('0')).right(width);
	QByteArray ret;
	for(int i=0 ; i<width ; ++i)
	{
		ret.append('\x82');
		ret.append((char)(chaine.at(i).digitValue()+0x4f));
	}
	return ret;
}

const quint8 FF8Text::width[195] =
{
	8, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14, 20, 12, 6, 8, 12, 18, 12, 12, 12, 16,
	16, 10, 10, 8, 8, 6, 6, 6, 16, 12, 12, 6, 16, 16, 6, 16, 14, 14, 14, 14, 14,
	14, 14, 14, 8, 12, 14, 12, 16, 14, 14, 14, 16, 14, 14, 16, 14, 16, 16, 16, 16, 16,
	14, 12, 12, 12, 12, 8, 12, 12, 4, 8, 12, 4, 16, 12, 12, 12, 12, 10, 12, 10, 12,
	12, 12, 12, 14, 12, 14, 14, 14, 14, 14, 14, 14, 14, 14, 6, 6, 12, 12, 14, 14, 14,
	14, 14, 14, 14, 14, 14, 16, 12, 14, 14, 14, 14, 12, 12, 12, 12, 12, 8, 8, 12, 12,
	12, 12, 12, 12, 12, 12, 12, 12, 12, 16, 24, 8, 8, 16, 16, 16, 10, 10, 16, 20, 12,
	12, 16, 6, 16, 16, 12, 20, 6, 12, 8, 8, 12, 24, 18, 18, 12, 16, 99, 12, 16, 12,
	8, 16, 18, 18, 16, 16, 20, 16, 16, 16, 12, 16, 16, 16, 16, 16, 12, 16, 24, 16, 16,
	16, 16, 16, 16, 16, 16
};
