/****************************************************************************
 **
 ** Copyright (C) 2010 Arzel Jrme <myst6re@gmail.com>
 **
 ** This program is free software: you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
 ** the Free Software Foundation, either version 3 of the License, or
 ** (at your option) any later version.
 **
 ** This program is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
 ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
 ** along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ****************************************************************************/

#include "AllEditor.h"

AllEditor::AllEditor(QWidget *parent) : QWidget(parent)
{
	buildWidget();
}

void AllEditor::load(MAIN *data, HEADER *descData)
{
	this->data = data;
	this->descData = descData;
	fillPage(tabBar->currentIndex());
}

void AllEditor::buildWidget()
{
	tabBar = new QTabBar(this);
	tabBar->addTab(tr("Tout"));
//	tabBar->addTab(tr("En-tte"));
	tabBar->addTab(tr("Divers 1"));
	tabBar->addTab(tr("Divers 2"));
	tabBar->addTab(tr("Inconnu"));
	tabBar->addTab(tr("Donnes field"));
	tabBar->addTab(tr("Chocobo World"));
//	tabBar->addTab(tr("G-Forces"));
//	tabBar->addTab(tr("Personnages"));
//	tabBar->addTab(tr("Limit Break"));
//	tabBar->addTab(tr("Objets"));
//	tabBar->addTab(tr("Config"));
	hexEdit = new QHexEdit(this);

	QVBoxLayout *dLayout = new QVBoxLayout(this);
	dLayout->addWidget(tabBar);
	dLayout->addWidget(hexEdit);
	dLayout->setContentsMargins(QMargins());

	connect(tabBar, SIGNAL(currentChanged(int)), SLOT(fillPage(int)));
	connect(hexEdit, SIGNAL(dataChanged()), SLOT(savePage()));
}

void AllEditor::fillPage(int id)
{
	char *cur;
	if(id==-1)	id = tabBar->currentIndex();

	switch(id) {
//	case 1:// Header
//		hexEdit->setData(QByteArray((char *)descData, sizeof(*descData)));
//		break;
	case 1:// Misc 1
		cur = (char *)data + 2724;
		hexEdit->setData(QByteArray(cur, 32));
		break;
	case 2:// Misc 2
		cur = (char *)data + 3200;
		hexEdit->setData(QByteArray(cur, 144));
		break;
	case 3:// Unknown 1
		cur = (char *)data + 2304;
		hexEdit->setData(QByteArray(cur, 400));
		break;
	case 4:// Field Vars
		cur = (char *)data + 3344;
		hexEdit->setData(QByteArray(cur, 1536));
		break;
	case 5:// Chocobo World
		cur = (char *)data + 4880;
		hexEdit->setData(QByteArray(cur, 64));
		break;
	/*case 6:// GF
		hexEdit->setData(QByteArray((char *)data, 1088));
		break;
	case 7:// Characters
		cur = (char *)data + 1088;
		hexEdit->setData(QByteArray(cur, 1216));
		break;
	case 8:// Limit Break
		cur = (char *)data + 2756;
		hexEdit->setData(QByteArray(cur, 16));
		break;
	case 9:// Item
		cur = (char *)data + 2772;
		hexEdit->setData(QByteArray(cur, 428));
		break;
	case 10:// Config
		cur = (char *)data + 2704;
		hexEdit->setData(QByteArray(cur, 20));
		break;*/
	default:// All
		hexEdit->setData(QByteArray(/*(char *)descData, sizeof(*descData)).append(*/(char *)data, sizeof(*data)));
		break;
	}
}

void AllEditor::savePage()
{
	const char *hexdata = hexEdit->data().constData();
	char *cur;

	switch(tabBar->currentIndex()) {
//	case 1:// Header
//		memcpy(descData, hexdata, sizeof(*descData));
//		break;
	case 1:// Misc 1
		cur = (char *)data + 2724;
		memcpy(cur, hexdata, 32);
		break;
	case 2:// Misc 2
		cur = (char *)data + 3200;
		memcpy(cur, hexdata, 144);
		break;
	case 3:// Unknown 1
		cur = (char *)data + 2304;
		memcpy(cur, hexdata, 400);
		break;
	case 4:// Field Vars
		cur = (char *)data + 3344;
		memcpy(cur, hexdata, 1536);
		break;
	case 5:// Chocobo World
		cur = (char *)data + 4880;
		memcpy(cur, hexdata, 64);
		break;
	/*case 6:// GF
		memcpy(data, hexdata, 1088);
		break;
	case 7:// Characters
		cur = (char *)data + 1088;
		memcpy(cur, hexdata, 1216);
		break;
	case 8:// Limit Break
		cur = (char *)data + 2756;
		memcpy(cur, hexdata, 16);
		break;
	case 9:// Item
		cur = (char *)data + 2772;
		memcpy(cur, hexdata, 428);
		break;
	case 10:// Config
		cur = (char *)data + 2704;
		memcpy(&data[2704], hexdata, 20);
		break;*/
	default:// All
//		memcpy(descData, hexdata, sizeof(*descData));
//		hexdata += sizeof(*descData);
		memcpy(data, hexdata, sizeof(*data));
		break;
	}
}
