/****************************************************************************
 **
 ** Copyright (C) 2010 Arzel Jrme <myst6re@gmail.com>
 **
 ** This program is free software: you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
 ** the Free Software Foundation, either version 3 of the License, or
 ** (at your option) any later version.
 **
 ** This program is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
 ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
 ** along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ****************************************************************************/

#include "Window.h"

Window::Window()
	: QWidget(), currentSaveEdited(0), isPCSlot(false), isOpen(false)
{
	setTitle();
	setMinimumSize(768, 502);
	resize(Config::valueVar("window_size", QSize(768, 502)).toSize());
	if(Config::valueVar("window_maximized", false).toBool()) {
		setWindowState(Qt::WindowMaximized);
	}
	setWindowIcon(QIcon(":/images/ico.png"));
	setAcceptDrops(true);
	
	menuBar = new QMenuBar(this);
	QMenu *menu;
	QAction *action;

	/* MENU 'FICHIER' */
	
	menu = menuBar->addMenu(tr("&Fichier"));
	
	menu->addAction(QIcon(":/images/open.png"), tr("&Ouvrir..."), this, SLOT(open()), QKeySequence(QKeySequence::Open));
	actionSaveAs = menu->addAction(tr("E&xporter..."), this, SLOT(saveAs()), QKeySequence(QKeySequence::SaveAs));
	actionSaveAs->setEnabled(false);
	menuRecent = menu->addMenu(tr("&Ouverts rcemment"));
	fillMenuRecent();
	connect(menuRecent, SIGNAL(triggered(QAction*)), SLOT(openRecentFile(QAction*)));
	menu->addSeparator();
	actionClose = menu->addAction(tr("&Fermer"), this, SLOT(closeFile()), QKeySequence(QKeySequence::Close));
	actionClose->setEnabled(false);
	menu->addAction(tr("&Quitter"), this, SLOT(close()), QKeySequence(QKeySequence::Quit));
	
	/* MENU 'SLOT' */
	
	bool isInstalled = !Config::ff8Path().isEmpty();
	
	action = menuBar->addAction(tr("Slot &1"), this, SLOT(slot1()));
	action->setEnabled(isInstalled);
	action = menuBar->addAction(tr("Slot &2"), this, SLOT(slot2()));
	action->setEnabled(isInstalled);
	
	/* MENU 'PARAMETRES' */
	
	menu = menuBar->addMenu(tr("&Paramtres"));
	
	actionMode = menu->addAction(tr("Mode Avanc"), this, SLOT(mode(bool)));
	actionMode->setCheckable(true);
	actionMode->setChecked(Config::mode());
	
	menuFrame = menu->addMenu(tr("Images par seconde"));
		action = menuFrame->addAction(tr("Auto"));
		action->setCheckable(true);
		action->setChecked(Config::freq_auto());
		action->setData(QVariant(0));
		action = menuFrame->addAction(tr("NTSC/PC (60 images/s)"));
		action->setCheckable(true);
		action->setChecked(Config::freq()==60);
		action->setData(QVariant(60));
		action = menuFrame->addAction(tr("PAL (50 images/s)"));
		action->setCheckable(true);
		action->setChecked(Config::freq()==50);
		action->setData(QVariant(50));
	connect(menuFrame, SIGNAL(triggered(QAction*)), SLOT(change_frame(QAction*)));
	
	actionFont = menu->addAction(tr("Police haute rsolution"), this, SLOT(font(bool)));
	actionFont->setCheckable(true);
	actionFont->setChecked(!Config::value("font").isEmpty());
	
	menuLang = menu->addMenu(tr("&Langues"));
		QDir dir(qApp->applicationDirPath());
		QStringList stringList = dir.entryList(QStringList("hyne_*.qm"), QDir::Files, QDir::Name);
		action = menuLang->addAction(tr("Franais (dfaut)"));
		action->setData("fr");
		action->setCheckable(true);
		action->setChecked(Config::value("lang")=="fr");

		menuLang->addSeparator();
		QTranslator translator;
		foreach(QString str, stringList) {
			translator.load(qApp->applicationDirPath()+"/"+str);
			action = menuLang->addAction(translator.translate("Window", "Franais"));
			str = str.mid(5,2);
			action->setData(str);
			action->setCheckable(true);
			action->setChecked(Config::value("lang")==str);
		}
	connect(menuLang, SIGNAL(triggered(QAction*)), SLOT(change_language(QAction*)));
	
	/* MENU '?' */
	
	menuBar->addAction(tr("?"), this, SLOT(a_propos()));
	
	blackView = new QListWidget(this);
	blackView->setPalette(QPalette(QColor(0,0,0)));
	
	editor = new Editor(this);
	editor->hide();

	connect(editor, SIGNAL(accepted()), SLOT(save()));
	connect(editor, SIGNAL(rejected()), SLOT(saveView()));
	
	stackedLayout = new QStackedLayout;
	stackedLayout->addWidget(blackView);
	stackedLayout->addWidget(editor);
	
	QVBoxLayout *gridLayout = new QVBoxLayout(this);
	gridLayout->setMenuBar(menuBar);
	gridLayout->addLayout(stackedLayout, 1);
	gridLayout->setContentsMargins(QMargins());
}

Window::~Window()
{
	Config::setValue("window_maximized", windowState()==Qt::WindowMaximized);
	if(windowState()!=Qt::WindowMaximized)
		Config::setValue("window_size", size());
	Config::saveRecentFiles();// hidden sync
}

void Window::showEvent(QShowEvent *)
{
	QRect screen = QApplication::desktop()->screenGeometry(this);
	move(screen.center().x() - width()/2, screen.center().y() - height()/2);
}

void Window::closeEvent(QCloseEvent *event)
{
	if(stackedLayout->currentWidget()==editor) {
		saveView();
		event->ignore();
	}
	else
		event->accept();
}

void Window::dragEnterEvent(QDragEnterEvent *event)
{
	if(stackedLayout->currentWidget()==blackView
	   || stackedLayout->currentWidget()==saves)
		event->acceptProposedAction();
}

void Window::dropEvent(QDropEvent *event)
{
	if(stackedLayout->currentWidget()==editor)	return;

	const QMimeData *mimeData = event->mimeData();

	if (mimeData->hasUrls())
	{
		QList<QUrl> urlList = mimeData->urls();
		if(!urlList.isEmpty()) {
			QString path = QDir::cleanPath(urlList.first().path());
			if(!path.startsWith(QDir::rootPath()))
				path = path.mid(1);
			if(path.startsWith(QDir::rootPath())) {
				this->isPCSlot = false;
				openFile(path);
			}
		}
	}

	event->acceptProposedAction();
}

void Window::setTitle(const bool editor)
{
	setWindowTitle(PROG_FULLNAME %
				   (Data::currentPath.isEmpty() ? QString() : QString(" - ")%Data::currentPath) %
				   (editor ? tr(" - slot %1").arg(currentSaveEdited+1, 2, 10, QChar('0')) : QString())
				   );
}

void Window::slot1()
{
	open(SLOT1);
}

void Window::slot2()
{
	open(SLOT2);
}

void Window::open(quint8 slot)
{
	QString path = Config::ff8Path();

	if(slot==SLOT1 || slot==SLOT2)
	{
		if(path.isEmpty())	return;

		path.append(QString("/Save/Slot%1/").arg(slot));

		this->isPCSlot = true;
	}
	else
	{
		if(!Config::value("loadPath").isEmpty())
			path = Config::value("loadPath");
		else if(!path.isEmpty())
			path.append("/Save");
		path = QFileDialog::getOpenFileName(this, tr("Ouvrir"), path, tr("Fichiers compatibles (*.mcr *.ddf *.gme *.mc *.mcd *.mci *.ps *.psm *.psv save?? *.mem *.vgs *.vmp);;FF8 PS memorycard (*.mcr *.ddf *.mc *.mcd *.mci *.ps *.psm);;FF8 PC save (save??);;FF8 vgs memorycard (*.mem *.vgs);;FF8 gme memorycard (*.gme);;FF8 PSN memorycard (*.vmp);;FF8 PS3 memorycard (*.psv);;Tous les fichiers (*)"));
		if(path==NULL)		return;

		int index = path.lastIndexOf('/');
		Config::setValue("loadPath", index == -1 ? path : path.left(index));

		this->isPCSlot = false;
	}
	
	openFile(path);
}

void Window::closeFile()
{
	stackedLayout->setCurrentWidget(blackView);
    editor->hide();
	if(this->isOpen)
    {
		isOpen = false;
		delete saves;
    }
    currentSaveEdited = 0;
    actionClose->setEnabled(false);
    actionSaveAs->setEnabled(false);
	Data::currentPath = QString();
	setTitle();
}

void Window::openFile(QString path)
{
	path = QDir::cleanPath(path);
	closeFile();
	this->saves = new Savecard(path, this, this->isPCSlot);

	Data::currentPath = path;

	this->isOpen = true;

	if(!saves->ok())
		closeFile();
	else
	{
		stackedLayout->addWidget(saves);
		stackedLayout->setCurrentWidget(saves);
		actionSaveAs->setEnabled(true);
		actionClose->setEnabled(true);
		setTitle();

		if(!isPCSlot) {
			Config::addRecentFile(path);
			fillMenuRecent();
		}
	}
}

void Window::openRecentFile(QAction *action)
{
	this->isPCSlot = false;
	openFile( Config::recentFile(action->data().toUInt()) );
}

bool Window::saveAs()
{
	QString types, path, selectedFilter,
			ps = tr("PS memorycard (*.mcr *.ddf *.mc *.mcd *.mci *.ps *.psm)"),
			vgs = tr("VGS memorycard (*.vgs *.mem)"),
			gme = tr("GME memorycard (*.gme)"),
			vmp = tr("PSN memorycard (*.vmp)"),
			pc = tr("FF8 PC save (*)"),
			psv = tr("PSN save (*.psv)");
	int type = saves->type();

	if(type == PCDIR || type == PC)
		types = pc+";;"+ps+";;"+vgs+";;"+gme+";;"+vmp;
	else if(type == PSV)
		types = psv+";;"+ps+";;"+vgs+";;"+gme+";;"+vmp;
	else
	{
		types = ps+";;"+vgs+";;"+gme+";;"+vmp;
		if(type == PS)		selectedFilter = ps;
		else if(type == VGS)	selectedFilter = vgs;
		else if(type == GME)	selectedFilter = gme;
		else if(type == VMP)	selectedFilter = vmp;
	}
	
	path = Config::value("savePath").isEmpty() ? saves->chemin() : Config::value("savePath")+"/";
	path = QFileDialog::getSaveFileName(this, tr("Exporter"), path+saves->name(), types, &selectedFilter);
	if(path==NULL)		return false;

	int index = path.lastIndexOf('.');
	QString extension = index != -1 ? path.mid(index+1).toLower() : "";

	if(extension == "vmp" || extension == "psv") {
		int reponse = QMessageBox::information(this, tr("Sauvegarde hasardeuse"), tr("Le format %1 est protg, l'enregistrement sera partiel et risque de ne pas fonctionner.\nContinuer quand mme ?").arg(extension), tr("Oui"), tr("Non"));
		if(reponse != 0)  return saveAs();
	}
	
	index = path.lastIndexOf('/');
	Config::setValue("savePath", index == -1 ? path : path.left(index));

	if(extension=="mcr" || extension=="ddf" || extension=="mc" || extension=="mcd" || extension=="mci" || extension=="ps" || extension=="psm"
	   || extension=="vgs" || extension=="mem" || extension=="gme" || extension=="vmp")
	{
		if(type == PCDIR) {
			QList<int> selected_files = selectSavesDialog(true);
			if(selected_files.isEmpty())	return false;
			saves->sauver2PS(selected_files, path);
		}
		else if(type == PC || type == PSV)
			saves->sauver2PS(QList<int>() << 0, path);
		else
			saves->sauver(path);
	}
	else
	{
		if(type == PCDIR) {
			QList<int> selected_files = selectSavesDialog();
			if(selected_files.isEmpty())	return false;
			int id = selected_files.first();
			saves->setName(QString("save%1").arg(id+1, 2, 10, QChar('0')));
			saves->sauverUn(id, path);
		}
		else if(type == PC)
			saves->sauverUn(0, path);
		else
			saves->sauver(path);
	}

	return true;
}

QList<int> Window::selectSavesDialog(bool multiSelection)
{
	QList<int> selected_saves;
	QDialog dialog(this, Qt::Dialog | Qt::WindowCloseButtonHint);

	QLabel msg(&dialog);
	QListWidget list(&dialog);
	if(multiSelection) {
		msg.setText(tr("Slectionnez jusqu' 15 slots  placer\n dans le nouveau fichier memory card :"));
		list.setSelectionMode(QAbstractItemView::ExtendedSelection);
	}
	else {
		msg.setText(tr("Slectionnez 1 slot  exporter :"));
	}
	QList<SaveData *> saveFiles = saves->getSaves();
	SaveData *save;
	QListWidgetItem *item;
	for(int i=0 ; i<30 ; ++i) {
		save = saveFiles.at(i);
		item = new QListWidgetItem(QString("Slot %1 (%2)").arg(i+1).arg(save->getShortDescription()));
		if(!save->isFF8())
			item->setFlags(Qt::NoItemFlags);
		list.addItem(item);
	}

	QPushButton ok(tr("OK"), &dialog);
	connect(&ok, SIGNAL(released()), &dialog, SLOT(accept()));

	QVBoxLayout layout(&dialog);
	layout.addWidget(&msg);
	layout.addWidget(&list);
	layout.addWidget(&ok, 0, Qt::AlignCenter);

	if(dialog.exec()==QDialog::Accepted) {
		foreach(QListWidgetItem *item, list.selectedItems())
			selected_saves.append(list.row(item));
	}
	return selected_saves;
}

void Window::fillMenuRecent()
{
	menuRecent->clear();
	
	for(int i=0 ; i<Config::recentFilesSize() ; ++i)
		menuRecent->addAction(QString("%1: "+Config::recentFile(i)).arg(i+1,2,10))->setData(i);

	menuRecent->setEnabled(Config::hasRecentFiles());
}

void Window::editView(SaveData *saveData)
{
	menuBar->hide();
	setWindowIcon(QIcon(saveData->icon()));
	editor->charger(saveData);
	currentSaveEdited = saveData->id();
	stackedLayout->setCurrentWidget(editor);
	editor->show();
	setTitle(true);
}

void Window::saveView()
{
	menuBar->show();
	setWindowIcon(QIcon(":/images/ico.png"));
	setTitle();
	editor->hide();
	stackedLayout->setCurrentWidget(saves);
}

void Window::save()
{
	bool saved = true;

	if(this->isPCSlot)
	{
		saves->sauverDossier(currentSaveEdited);
	}
	else
	{
		int type = saves->type();
		if(type == VMP || type == PSV)
			saved = saveAs();
		else if(type == PC)
			saves->sauverUn(0);
		else
			saves->sauver();
	}
	if(saved) {
		saveView();
		saves->updateSaveWidget(currentSaveEdited, true);
	}
}

void Window::mode(bool mode)
{
	Config::setValue("mode", mode);
	if(this->isOpen)
	{
		editor->updateMode();
	}
}

void Window::change_frame(QAction *action)
{
	if(action->data().toInt()==0)
		Config::setValue("freq_auto", QVariant(true));
	else {
		Config::setValue("freq", action->data());
		Config::setValue("freq_auto", QVariant(false));
	}

	foreach(QAction *act, menuFrame->actions())
		act->setChecked(false);
	action->setChecked(true);

	if(this->isOpen)
	{
		editor->updateTime();
		saves->updateSaveWidgets();
	}
}

void Window::font(bool font)
{
	Config::setValue("font", font ? "hr" : "");
	if(this->isOpen)
	{
		saves->updateSaveWidgets();
	}
}

void Window::change_language(QAction *action)
{
	Config::setValue("lang", action->data());
	foreach(QAction *act, menuLang->actions())
		act->setChecked(false);
	action->setChecked(true);

	restart_now();
}

void Window::restart_now()
{
    QString str_title, str_text;
    QTranslator translator;
	if(translator.load(qApp->applicationDirPath() % QString("/hyne_") % Config::value("lang"))) {
        str_title = translator.translate("Window", "Paramtres modifis");
        str_text = translator.translate("Window", "Relancez le programme pour que les paramtres prennent effet.");
    }
    else {
        str_title = "Paramtres modifis";
        str_text = "Relancez le programme pour que les paramtres prennent effet.";
    }
    QMessageBox::information(this, str_title, str_text);
}

void Window::a_propos()
{
	QDialog apropos(this, Qt::Dialog | Qt::CustomizeWindowHint);
	apropos.setFixedSize(200, 224);
	
	QFont font;
	font.setPointSize(12);
	
	QLabel image(&apropos);
	image.setPixmap(QPixmap(":/images/nacre_blade.png"));
	image.move(104, 49);
	
	QLabel desc1(PROG_FULLNAME, &apropos);
	desc1.setFont(font);
	desc1.setFixedWidth(200);
	desc1.setAlignment(Qt::AlignHCenter);

	font.setPointSize(8);

	QLabel desc2(tr("Par myst6re\nmyst6re@gmail.com\n\n60% modifiable\n\nMerci  :\n - Qhimm\n - Cyberman\n - sithlord48\n - Aladore384"), &apropos);
	desc2.move(9, 40);
	desc2.setFont(font);

	QLabel desc3(QString("<font color=\"#AAA\">Qt %1</font>").arg(qVersion()), &apropos);
	desc3.move(192-desc3.sizeHint().width(), 216-desc3.sizeHint().height());
	desc3.setFont(font);
	
	QPushButton button(tr("Fermer"), &apropos);
	button.move(8, 193);
	connect(&button, SIGNAL(released()), &apropos, SLOT(close()));
	
	apropos.exec();
}
